@@ -47,7 +47,6 @@
 
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-browser.h>
-#include <thunar/thunar-create-dialog.h>
 #include <thunar/thunar-dialogs.h>
 #include <thunar/thunar-gdk-extensions.h>
 #include <thunar/thunar-gobject-extensions.h>
@@ -142,9 +141,11 @@ static void           thunar_application_collect_and_launch     (ThunarApplicati
                                                                  Launcher                launcher,
                                                                  GList                  *source_file_list,
                                                                  GFile                  *target_file,
+                                                                 gboolean                update_source_folders,
+                                                                 gboolean                update_target_folders,
                                                                  GClosure               *new_files_closure);
 static void           thunar_application_launch_finished        (ThunarJob              *job,
-                                                                 ThunarView             *view);
+                                                                 GList                  *containing_folders);
 static void           thunar_application_launch                 (ThunarApplication      *application,
                                                                  gpointer                parent,
                                                                  const gchar            *icon_name,
@@ -152,6 +153,8 @@ static void           thunar_application_launch                 (ThunarApplicati
                                                                  Launcher                launcher,
                                                                  GList                  *source_path_list,
                                                                  GList                  *target_path_list,
+                                                                 gboolean                update_source_folders,
+                                                                 gboolean                update_target_folders,
                                                                  GClosure               *new_files_closure);
 #ifdef HAVE_GUDEV
 static void           thunar_application_uevent                 (GUdevClient            *client,
@@ -179,38 +182,37 @@ struct _ThunarApplicationClass
 
 struct _ThunarApplication
 {
-  GtkApplication                 __parent__;
+  GtkApplication         __parent__;
 
-  ThunarSessionClient            *session_client;
+  ThunarSessionClient   *session_client;
 
-  ThunarPreferences              *preferences;
-  GtkWidget                      *progress_dialog;
+  ThunarPreferences     *preferences;
+  GtkWidget             *progress_dialog;
 
-  ThunarThumbnailCache           *thumbnail_cache;
-  ThunarThumbnailer              *thumbnailer;
+  ThunarThumbnailCache  *thumbnail_cache;
+  ThunarThumbnailer     *thumbnailer;
 
-  ThunarDBusService              *dbus_service;
+  ThunarDBusService     *dbus_service;
 
-  gboolean                        daemon;
+  gboolean               daemon;
 
-  guint                           accel_map_save_id;
-  GtkAccelMap                    *accel_map;
+  guint                  accel_map_save_id;
+  GtkAccelMap           *accel_map;
 
-  guint                           show_dialogs_timer_id;
+  guint                  show_dialogs_timer_id;
 
 #ifdef HAVE_GUDEV
-  GUdevClient                    *udev_client;
+  GUdevClient           *udev_client;
 
-  GSList                         *volman_udis;
-  guint                           volman_idle_id;
-  guint                           volman_watch_id;
+  GSList                *volman_udis;
+  guint                  volman_idle_id;
+  guint                  volman_watch_id;
 #endif
 
-  GList                          *files_to_launch;
-  ThunarApplicationProcessAction  process_file_action;
+  GList                 *files_to_launch;
 
-  guint                           dbus_owner_id_xfce;
-  guint                           dbus_owner_id_fdo;
+  guint                  dbus_owner_id_xfce;
+  guint                  dbus_owner_id_fdo;
 };
 
 
@@ -277,7 +279,6 @@ thunar_application_init (ThunarApplication *application)
    * in the primary instance anyways */
 
   application->files_to_launch = NULL;
-  application->process_file_action = THUNAR_APPLICATION_SELECT_FILES;
   application->progress_dialog = NULL;
   application->preferences     = NULL;
 
@@ -372,7 +373,6 @@ thunar_application_startup (GApplication *gapp)
   /* connect to the session manager */
   application->session_client = thunar_session_client_new (opt_sm_client_id);
 
-  /* TODO: how do accel maps integrate with GAction/GMenu? Using GtkAction for now */
   /* check if we have a saved accel map */
   path = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, ACCEL_MAP_PATH);
   if (G_LIKELY (path != NULL))
@@ -471,7 +471,7 @@ thunar_application_handle_local_options (GApplication *gapp,
   if (G_UNLIKELY (opt_version))
     {
       g_print ("%s %s (Xfce %s)\n\n", PACKAGE_NAME, PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2004-2019");
+      g_print ("%s\n", "Copyright (c) 2004-2020");
       g_print ("\t%s\n\n", _("The Thunar development team. All rights reserved."));
       g_print ("%s\n\n", _("Written by Benedikt Meurer <benny@xfce.org>."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
@@ -531,7 +531,7 @@ thunar_application_command_line (GApplication            *gapp,
     }
   else if (filenames != NULL)
     {
-      if (!thunar_application_process_filenames (application, cwd, filenames, NULL, NULL, &error, THUNAR_APPLICATION_SELECT_FILES))
+      if (!thunar_application_process_filenames (application, cwd, filenames, NULL, NULL, &error))
         {
           /* we failed to process the filenames or the bulk rename failed */
           g_application_command_line_printerr (command_line, "Thunar: %s\n", error->message);
@@ -539,7 +539,7 @@ thunar_application_command_line (GApplication            *gapp,
     }
   else if (!daemon)
     {
-      if (!thunar_application_process_filenames (application, cwd, cwd_list, NULL, NULL, &error, THUNAR_APPLICATION_SELECT_FILES))
+      if (!thunar_application_process_filenames (application, cwd, cwd_list, NULL, NULL, &error))
         {
           /* we failed to process the filenames or the bulk rename failed */
           g_application_command_line_printerr (command_line, "Thunar: %s\n", error->message);
@@ -709,6 +709,8 @@ thunar_application_collect_and_launch (ThunarApplication *application,
                                        Launcher           launcher,
                                        GList             *source_file_list,
                                        GFile             *target_file,
+                                       gboolean           update_source_folders,
+                                       gboolean           update_target_folders,
                                        GClosure          *new_files_closure)
 {
   GFile  *file;
@@ -755,7 +757,7 @@ thunar_application_collect_and_launch (ThunarApplication *application,
     {
       /* launch the operation */
       thunar_application_launch (application, parent, icon_name, title, launcher,
-                                 source_file_list, target_file_list, new_files_closure);
+                                 source_file_list, target_file_list, update_source_folders, update_target_folders, new_files_closure);
     }
 
   /* release the target path list */
@@ -764,38 +766,41 @@ thunar_application_collect_and_launch (ThunarApplication *application,
 
 
 
-static void
-thunar_application_launch_finished_too_late (gpointer  user_data,
-                                             GObject  *where_the_object_was)
-{
-  ThunarJob *job = THUNAR_JOB (user_data);
-
-  _thunar_return_if_fail (THUNAR_IS_JOB (job));
-  _thunar_return_if_fail (THUNAR_IS_VIEW (where_the_object_was));
-
-  /* remove the finished signal */
-  g_signal_handlers_disconnect_by_func (G_OBJECT (job),
-                                        G_CALLBACK (thunar_application_launch_finished),
-                                        where_the_object_was);
-}
-
-
-
 static void
 thunar_application_launch_finished (ThunarJob  *job,
-                                    ThunarView *view)
+                                    GList      *containing_folders)
 {
+  GList        *lp;
+  ThunarFile   *file;
+  ThunarFolder *folder;
+
   _thunar_return_if_fail (THUNAR_IS_JOB (job));
-  _thunar_return_if_fail (THUNAR_IS_VIEW (view));
 
-  /* remove the view weak ref */
-  g_object_weak_unref (G_OBJECT (view),
-                       thunar_application_launch_finished_too_late,
-                       job);
-
-  /* the job completed, refresh the interface
-   * directly to make it feel snappy */
-  thunar_view_reload (view, FALSE);
+  for (lp = containing_folders; lp != NULL; lp = lp->next)
+    {
+      if (lp->data == NULL)
+        continue;
+      file = thunar_file_get (lp->data, NULL);
+      if (file != NULL)
+        {
+          if (thunar_file_is_directory (file))
+            {
+              folder = thunar_folder_get_for_file (file);
+              if (folder != NULL)
+                {
+                  /* If the folder is connected to a folder monitor, we dont need to trigger the reload manually */
+                  if (!thunar_folder_has_folder_monitor (folder))
+                  {
+                    thunar_folder_reload (folder, FALSE);
+                  }
+                  g_object_unref (folder);
+                }
+            }
+          g_object_unref (file);
+        }
+      /* Unref all containing_folders (refs obtained by g_file_get_parent in thunar_g_file_list_get_parents ) */
+      g_object_unref (lp->data);
+    }
 }
 
 
@@ -808,11 +813,15 @@ thunar_application_launch (ThunarApplication *application,
                            Launcher           launcher,
                            GList             *source_file_list,
                            GList             *target_file_list,
+                           gboolean           update_source_folders,
+                           gboolean           update_target_folders,
                            GClosure          *new_files_closure)
 {
   GtkWidget *dialog;
   GdkScreen *screen;
   ThunarJob *job;
+  GList     *parent_folder_list = NULL;
+  gboolean   has_jobs;
 
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
 
@@ -822,19 +831,15 @@ thunar_application_launch (ThunarApplication *application,
   /* try to allocate a new job for the operation */
   job = (*launcher) (source_file_list, target_file_list);
 
-  if (THUNAR_IS_VIEW (parent))
-    {
-      /* connect a callback to instantly refresh the thunar view */
-      g_signal_connect (G_OBJECT (job), "finished",
-                        G_CALLBACK (thunar_application_launch_finished),
-                        parent);
+  if (update_source_folders)
+    parent_folder_list = g_list_concat (parent_folder_list, thunar_g_file_list_get_parents (source_file_list));
+  if (update_target_folders)
+    parent_folder_list = g_list_concat (parent_folder_list, thunar_g_file_list_get_parents (target_file_list));
 
-      /* watch destruction of the parent, so we disconnect before the
-       * job is finished */
-      g_object_weak_ref (G_OBJECT (parent),
-                         thunar_application_launch_finished_too_late,
-                         job);
-    }
+  /* connect a callback to instantly refresh the parent folders after the operation finished */
+  g_signal_connect (G_OBJECT (job), "finished",
+                    G_CALLBACK (thunar_application_launch_finished),
+                    parent_folder_list);
 
   /* connect the "new-files" closure (if any) */
   if (G_LIKELY (new_files_closure != NULL))
@@ -847,21 +852,19 @@ thunar_application_launch (ThunarApplication *application,
   if (screen != NULL)
     gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
-  if (thunar_progress_dialog_has_jobs (THUNAR_PROGRESS_DIALOG (dialog)))
-    {
-      /* add the job to the dialog */
-      thunar_progress_dialog_add_job (THUNAR_PROGRESS_DIALOG (dialog),
-                                      job, icon_name, title);
+  has_jobs = thunar_progress_dialog_has_jobs (THUNAR_PROGRESS_DIALOG (dialog));
 
+  /* add the job to the dialog */
+  thunar_progress_dialog_add_job (THUNAR_PROGRESS_DIALOG (dialog),
+                                  job, icon_name, title);
+
+  if (has_jobs)
+    {
       /* show the dialog immediately */
       thunar_application_show_dialogs (application);
     }
   else
     {
-      /* add the job to the dialog */
-      thunar_progress_dialog_add_job (THUNAR_PROGRESS_DIALOG (dialog),
-                                      job, icon_name, title);
-
       /* Set up a timer to show the dialog, to make sure we don't
        * just popup and destroy a dialog for a very short job.
        */
@@ -1509,27 +1512,8 @@ thunar_application_process_files_finish (ThunarBrowser *browser,
     }
   else
     {
-      if (application->process_file_action == THUNAR_APPLICATION_LAUNCH_FILES)
-        {
-          /* try to launch the file / open the directory */
-          thunar_file_launch (target_file, screen, startup_id, &error);
-        }
-      else if (thunar_file_is_directory (file))
-        {
-          thunar_application_open_window (application, file, screen, startup_id, FALSE);
-        }
-      else
-        {
-          /* Note that for security reasons we do not execute files passed via command line */
-          /* Lets rather open the containing directory */
-          ThunarFile *parent = thunar_file_get_parent (file, NULL);
-
-          if (G_LIKELY (parent != NULL))
-            {
-              thunar_application_open_window (application, parent, screen, startup_id, FALSE);
-              g_object_unref (parent);
-            }
-        }
+      /* try to open the file or directory */
+      thunar_file_launch (target_file, screen, startup_id, &error);
 
       /* remove the file from the list */
       application->files_to_launch = g_list_delete_link (application->files_to_launch,
@@ -1598,20 +1582,18 @@ thunar_application_process_files (ThunarApplication *application)
  * @startup_id        : startup id to finish startup notification and properly focus the
  *                      window when focus stealing is enabled or %NULL.
  * @error             : return location for errors or %NULL.
- * @action            : action to invoke on the files
  *
  * Tells @application to process the given @filenames and launch them appropriately.
  *
  * Return value: %TRUE on success, %FALSE if @error is set.
  **/
 gboolean
-thunar_application_process_filenames (ThunarApplication               *application,
-                                      const gchar                     *working_directory,
-                                      gchar                          **filenames,
-                                      GdkScreen                       *screen,
-                                      const gchar                     *startup_id,
-                                      GError                         **error,
-                                      ThunarApplicationProcessAction   action)
+thunar_application_process_filenames (ThunarApplication *application,
+                                      const gchar       *working_directory,
+                                      gchar            **filenames,
+                                      GdkScreen         *screen,
+                                      const gchar       *startup_id,
+                                      GError           **error)
 {
   ThunarFile *file;
   GError     *derror = NULL;
@@ -1683,10 +1665,7 @@ thunar_application_process_filenames (ThunarApplication               *applicati
 
   /* start processing files if we have any to launch */
   if (application->files_to_launch != NULL)
-    {
-      application->process_file_action = action;
-      thunar_application_process_files (application);
-    }
+    thunar_application_process_files (application);
 
   /* free the file list */
   g_list_free (file_list);
@@ -1828,7 +1807,7 @@ thunar_application_create_file (ThunarApplication *application,
   /* TODO pass the startup ID to the rename dialog */
 
   /* ask the user to enter a name for the new folder */
-  name = thunar_show_create_dialog (screen, content_type, dialog_title, title);
+  name = thunar_dialogs_show_create (screen, content_type, dialog_title, title);
   if (G_LIKELY (name != NULL))
     {
       path_list.data = g_file_get_child (thunar_file_get_file (parent_directory), name);
@@ -1885,10 +1864,10 @@ thunar_application_create_file_from_template (ThunarApplication *application,
   /* TODO pass the startup ID to the rename dialog */
 
   /* ask the user to enter a name for the new document */
-  name = thunar_show_create_dialog (screen,
-                                    thunar_file_get_content_type (template_file),
-                                    thunar_file_get_display_name (template_file),
-                                    title);
+  name = thunar_dialogs_show_create (screen,
+                                     thunar_file_get_content_type (template_file),
+                                     thunar_file_get_display_name (template_file),
+                                     title);
   if (G_LIKELY (name != NULL))
     {
       /* fake the target path list */
@@ -1944,7 +1923,7 @@ thunar_application_copy_to (ThunarApplication *application,
   /* launch the operation */
   thunar_application_launch (application, parent, "edit-copy",
                              _("Copying files..."), thunar_io_jobs_copy_files,
-                             source_file_list, target_file_list, new_files_closure);
+                             source_file_list, target_file_list, FALSE, TRUE, new_files_closure);
 }
 
 
@@ -1986,6 +1965,7 @@ thunar_application_copy_into (ThunarApplication *application,
   thunar_application_collect_and_launch (application, parent, "edit-copy",
                                          title, thunar_io_jobs_copy_files,
                                          source_file_list, target_file,
+                                         FALSE, TRUE,
                                          new_files_closure);
 
   /* free the title */
@@ -2032,6 +2012,7 @@ thunar_application_link_into (ThunarApplication *application,
   thunar_application_collect_and_launch (application, parent, "insert-link",
                                          title, thunar_io_jobs_link_files,
                                          source_file_list, target_file,
+                                         FALSE, TRUE,
                                          new_files_closure);
 
   /* free the title */
@@ -2086,6 +2067,7 @@ thunar_application_move_into (ThunarApplication *application,
                                              "stock_folder-move", title,
                                              thunar_io_jobs_move_files,
                                              source_file_list, target_file,
+                                             TRUE, TRUE,
                                              new_files_closure);
 
       /* free the title */
@@ -2196,7 +2178,7 @@ thunar_application_unlink_files (ThunarApplication *application,
           /* launch the "Delete" operation */
           thunar_application_launch (application, parent, "edit-delete",
                                      _("Deleting files..."), unlink_stub,
-                                     path_list, path_list, NULL);
+                                     path_list, path_list, TRUE, FALSE, NULL);
         }
     }
   else
@@ -2231,7 +2213,7 @@ thunar_application_trash (ThunarApplication *application,
 
   thunar_application_launch (application, parent, "user-trash-full",
                              _("Moving files into the trash..."), trash_stub,
-                             file_list, NULL, NULL);
+                             file_list, NULL, TRUE, FALSE, NULL);
 }
 
 
@@ -2277,7 +2259,7 @@ thunar_application_creat (ThunarApplication *application,
   /* launch the operation */
   thunar_application_launch (application, parent, "document-new",
                              _("Creating files..."), creat_stub,
-                             &template_list, file_list, new_files_closure);
+                             &template_list, file_list, FALSE, TRUE, new_files_closure);
 }
 
 
@@ -2316,7 +2298,7 @@ thunar_application_mkdir (ThunarApplication *application,
   /* launch the operation */
   thunar_application_launch (application, parent, "folder-new",
                              _("Creating directories..."), mkdir_stub,
-                             file_list, file_list, new_files_closure);
+                             file_list, file_list, TRUE, FALSE, new_files_closure);
 }
 
 
@@ -2382,7 +2364,7 @@ thunar_application_empty_trash (ThunarApplication *application,
       /* launch the operation */
       thunar_application_launch (application, parent, "user-trash",
                                  _("Emptying the Trash..."),
-                                 unlink_stub, &file_list, NULL, NULL);
+                                 unlink_stub, &file_list, NULL, TRUE, FALSE, NULL);
 
       /* cleanup */
       g_object_unref (file_list.data);
@@ -2453,7 +2435,7 @@ thunar_application_restore_files (ThunarApplication *application,
       /* launch the operation */
       thunar_application_launch (application, parent, "stock_folder-move",
                                  _("Restoring files..."), thunar_io_jobs_restore_files,
-                                 source_path_list, target_path_list, new_files_closure);
+                                 source_path_list, target_path_list, TRUE, TRUE, new_files_closure);
     }
 
   /* free path lists */
