@@ -557,13 +557,14 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
                                   GValue       *value)
 {
   ThunarShortcut *shortcut;
-  gboolean        can_eject;
   GFile          *file;
-  gchar          *disk_usage;
-  guint32         trash_items;
-  gchar          *trash_string;
-  gchar          *parse_name;
+  gboolean        can_eject;
   gboolean        file_size_binary;
+  gchar          *disk_usage;
+  gchar          *location;
+  gchar          *tooltip;
+  guint32         trash_items;
+  gchar          *parse_name;
 
   _thunar_return_if_fail (iter->stamp == THUNAR_SHORTCUTS_MODEL (tree_model)->stamp);
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (tree_model));
@@ -617,10 +618,27 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
 
           if (file != NULL)
             {
+              if (g_file_has_uri_scheme (file, "file") && thunar_g_file_is_root (file))
+                location = g_strdup (_("Browse the file system"));
+              else if (!g_file_has_uri_scheme (file, "file"))
+                location = g_file_get_uri (file);
+              else
+                location = thunar_g_file_get_location (file);
+
               file_size_binary = THUNAR_SHORTCUTS_MODEL (tree_model)->file_size_binary;
               disk_usage = thunar_g_file_get_free_space_string (file, file_size_binary);
+
+              if (disk_usage)
+                tooltip = g_strdup_printf ("%s\n%s", location, disk_usage);
+              else
+                tooltip = g_strdup_printf ("%s", location);
+
+              g_value_take_string (value, tooltip);
+
+              g_free (location);
+              g_free (disk_usage);
+
               g_object_unref (file);
-              g_value_take_string (value, disk_usage);
             }
           break;
         }
@@ -633,27 +651,39 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
             }
           else
             {
-              trash_string = g_strdup_printf (ngettext ("Trash contains %d file",
-                                                        "Trash contains %d files",
-                                                        trash_items), trash_items);
-              g_value_take_string (value, trash_string);
+              tooltip = g_strdup_printf (ngettext ("Trash contains %d file",
+                                                   "Trash contains %d files",
+                                                   trash_items), trash_items);
+              g_value_take_string (value, tooltip);
             }
           break;
         }
       else if (shortcut->tooltip == NULL)
         {
-          if (shortcut->file != NULL)
-            file = thunar_file_get_file (shortcut->file);
-          else if (shortcut->location != NULL)
-            file = shortcut->location;
-          else
-            file = NULL;
-
-          if (G_LIKELY (file != NULL))
+          if (shortcut->device != NULL)
             {
-              parse_name = g_file_get_parse_name (file);
-              shortcut->tooltip = g_markup_escape_text (parse_name, -1);
-              g_free (parse_name);
+              file = thunar_device_get_root (shortcut->device);
+              if (G_LIKELY (file != NULL))
+                {
+                  shortcut->tooltip = g_file_get_uri (file);
+                  g_object_unref (file);
+                }
+            }
+          else
+            {
+              if (shortcut->file != NULL)
+                file = thunar_file_get_file (shortcut->file);
+              else if (shortcut->location != NULL)
+                file = shortcut->location;
+              else
+                file = NULL;
+
+              if (G_LIKELY (file != NULL))
+                {
+                  parse_name = g_file_get_parse_name (file);
+                  shortcut->tooltip = g_markup_escape_text (parse_name, -1);
+                  g_free (parse_name);
+                }
             }
         }
 
@@ -927,13 +957,14 @@ thunar_shortcuts_model_shortcut_devices (ThunarShortcutsModel *model)
   /* add the devices heading */
   shortcut = g_slice_new0 (ThunarShortcut);
   shortcut->group = THUNAR_SHORTCUT_GROUP_DEVICES_HEADER;
-  shortcut->name = g_strdup (_("DEVICES"));
+  shortcut->name = g_strdup (_("Devices"));
   thunar_shortcuts_model_add_shortcut (model, shortcut);
 
   /* the filesystem entry */
   shortcut = g_slice_new0 (ThunarShortcut);
   shortcut->group = THUNAR_SHORTCUT_GROUP_DEVICES_FILESYSTEM;
   shortcut->name = g_strdup (_("File System"));
+  shortcut->tooltip = g_strdup (_("Browse the file system"));
   shortcut->file = thunar_file_get_for_uri ("file:///", NULL);
   shortcut->gicon = g_themed_icon_new ("drive-harddisk");
   shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
@@ -969,17 +1000,21 @@ thunar_shortcuts_model_shortcut_network (ThunarShortcutsModel *model)
   /* add the network heading */
   shortcut = g_slice_new0 (ThunarShortcut);
   shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_HEADER;
-  shortcut->name = g_strdup (_("NETWORK"));
+  shortcut->name = g_strdup (_("Network"));
   thunar_shortcuts_model_add_shortcut (model, shortcut);
 
-  /* the browse network entry */
-  shortcut = g_slice_new0 (ThunarShortcut);
-  shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_DEFAULT;
-  shortcut->name = g_strdup (_("Browse Network"));
-  shortcut->location = g_file_new_for_uri ("network://");
-  shortcut->gicon = g_themed_icon_new ("network-workgroup");
-  shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
-  thunar_shortcuts_model_add_shortcut (model, shortcut);
+  /* append the browse network entry if it is supported */
+  if (thunar_g_vfs_is_uri_scheme_supported ("network"))
+    {
+      shortcut = g_slice_new0 (ThunarShortcut);
+      shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_DEFAULT;
+      shortcut->name = g_strdup (_("Browse Network"));
+      shortcut->tooltip = g_strdup (_("Browse local network connections"));
+      shortcut->location = g_file_new_for_uri ("network://");
+      shortcut->gicon = g_themed_icon_new ("network-workgroup");
+      shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
+      thunar_shortcuts_model_add_shortcut (model, shortcut);
+    }
 }
 
 
@@ -996,7 +1031,7 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
   /* add the places heading */
   shortcut = g_slice_new0 (ThunarShortcut);
   shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_HEADER;
-  shortcut->name = g_strdup (_("PLACES"));
+  shortcut->name = g_strdup (_("Places"));
   thunar_shortcuts_model_add_shortcut (model, shortcut);
 
   /* get home path */
@@ -1008,6 +1043,7 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
     {
       shortcut = g_slice_new0 (ThunarShortcut);
       shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_DEFAULT;
+      shortcut->tooltip = g_strdup (_("Open the home folder"));
       shortcut->file = file;
       shortcut->gicon = g_themed_icon_new ("go-home");
       shortcut->sort_id = 0;
@@ -1024,6 +1060,7 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
         {
           shortcut = g_slice_new0 (ThunarShortcut);
           shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_DEFAULT;
+          shortcut->tooltip = g_strdup (_("Open the desktop folder"));
           shortcut->file = file;
           shortcut->sort_id =  1;
           shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
@@ -1064,6 +1101,19 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
 
   /* read the Gtk+ bookmarks file */
   model->bookmarks_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_load, model, NULL);
+
+  /* append the computer icon if browsing the computer is supported */
+  if (thunar_g_vfs_is_uri_scheme_supported ("computer"))
+    {
+      shortcut = g_slice_new0 (ThunarShortcut);
+      shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_COMPUTER;
+      shortcut->name = g_strdup (_("Computer"));
+      shortcut->tooltip = g_strdup (_("Browse the computer"));
+      shortcut->location = g_file_new_for_uri ("computer://");
+      shortcut->gicon = g_themed_icon_new ("computer");
+      shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
+      thunar_shortcuts_model_add_shortcut (model, shortcut);
+    }
 }
 
 
@@ -1915,8 +1965,8 @@ thunar_shortcuts_model_drop_possible (ThunarShortcutsModel *model,
   if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
     return TRUE;
 
-  /* we can drop at the end of the bookmarks (before network header) */
-  if (shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_HEADER)
+  /* we can drop at the end of the bookmarks (before devices header) */
+  if (shortcut->group == THUNAR_SHORTCUT_GROUP_DEVICES_HEADER)
     return TRUE;
 
   return FALSE;
@@ -1940,6 +1990,8 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
 {
   ThunarShortcut *shortcut;
   GFile          *location;
+  GList          *lp;
+  guint           position = 0;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
   _thunar_return_if_fail (dst_path == NULL || gtk_tree_path_get_depth (dst_path) > 0);
@@ -1967,6 +2019,16 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
       shortcut->gicon = g_themed_icon_new ("folder-remote");
     }
 
+  /* if no position was given, place the shortcut at the bottom */
+  if (dst_path == NULL)
+    {
+      for (lp = model->shortcuts; lp != NULL; lp = lp->next)
+        if (THUNAR_SHORTCUT (lp->data)->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
+          position++;
+
+      shortcut->sort_id = ++position;
+    }
+
   /* add the shortcut to the list at the given position */
   thunar_shortcuts_model_add_shortcut_with_path (model, shortcut, dst_path);
 
